
function aafActuation = Actuation( tController, atDrones )
    
    switch( tController.sType )
        case 'VirtualFields'
            aafActuation = VirtualFieldsActuation( atDrones, 3*tController.fGain );
        case 'OpenBorders'
            aafActuation = OpenBordersActuation( atDrones, tController.fGain );
    end
    
    
end

function aafActuation = OpenBordersActuation( atDrones, fGain )
    aafActuation = zeros(2, numel(atDrones));
    for iDrone = 1:numel(atDrones)
        aafFreeAngles = atDrones(iDrone).GetFreeAngles();
        for iAngle = 1:size(aafFreeAngles,1)
            %ref thesis for integral over angles
            aafActuation(:,iDrone) = aafActuation(:, iDrone) + ...
                fGain*[sin(aafFreeAngles(iAngle,2)) - sin(aafFreeAngles(iAngle,1)); ...
                -cos(aafFreeAngles(iAngle,2))+cos(aafFreeAngles(iAngle,1))];
        end
    end
end

function aafActuation = VirtualFieldsActuation( atDrones, fGain )

    fComRange = 2*atDrones(1).tSensor.Range(2);
    
    aafActuation = zeros( 2, numel(atDrones) );
    for iDrone = 1:numel(atDrones)
        
        [afAngles, afMeasurements] = atDrones(iDrone).GetSensorReading();
        
        %forces from wall
        %adds last and first to also check local min in beginning and end. 
        abLocalMins = islocalmin([afMeasurements(end-1); afMeasurements]);
        abLocalMins = abLocalMins(2:end-1); 
        afObjectForce = [0;0];
        if any(abLocalMins)
            afDeltaX = cos(afAngles(abLocalMins)).*afMeasurements(abLocalMins);
            afDeltaY = sin(afAngles(abLocalMins)).*afMeasurements(abLocalMins);
            afDistance = sqrt(sum(afDeltaX.^2+afDeltaY.^2,2));
            afObjectForce = sum([afDeltaX'; afDeltaY']./((afDistance.^3)'),2);
            afObjectForce1 = sum([cos(afAngles(abLocalMins))'; sin(afAngles(abLocalMins))']./...
                (afMeasurements(abLocalMins)'*(afMeasurements(abLocalMins))),2);
        end
        afObjectForce = -fGain*afObjectForce;
        %forces from other drones
        afDroneForce = [0;0];
        for iOtherDrone = 1:numel(atDrones)
            if iOtherDrone == iDrone
                continue
            end

            afDistance = atDrones(iOtherDrone).afState(1:2)- atDrones(iDrone).afState(1:2);
            fDistMagnitude = sqrt(afDistance'*afDistance);
            if fDistMagnitude <= fComRange
                afDroneForce = afDroneForce + 1/fDistMagnitude^3 * afDistance;
            end
        end
        afDroneForce = -fGain*afDroneForce;
        aafActuation(:,iDrone) = afObjectForce + afDroneForce;
    end
end

