
% Update obj state of the drone using its system dynamics 
% defined during the construction of object
%
% Discretization is done using first order linearization
%
% Output:
% [xPos; yPos; xVel; yVel]

function afNewState = Actuate(tDrone, afInput, fTimestep)

    afDeltaX = fTimestep * tDrone.tDynamics.f(tDrone.afState, afInput);
    
    velSquared = afDeltaX(1:2)'*afDeltaX(1:2);
    if velSquared > tDrone.fMaxVel^2
        afDeltaX(1) = afDeltaX(1) *tDrone.fMaxVel/sqrt(velSquared);
        afDeltaX(2) = afDeltaX(2) *tDrone.fMaxVel/sqrt(velSquared);
    end

    % update using saturated deltaX
    afNewState = tDrone.afState + afDeltaX;
    afNewState(3:4) = afDeltaX(1:2);

%     if afNewState(3) > tDrone.fMaxVel
%         afNewState(3) = tDrone.fMaxVel;
%     end
%     if afNewState(4) > tDrone.fMaxVel
%         afNewState(4) = tDrone.fMaxVel;
%     end

    tDrone.afState = afNewState;

end
