
% Updates and return the new sensor reading based on the input environment
% Sensor is a range bearing, where index represents steps of angle and
% element is the range
%

function afSensorReading = UpdateSensorReading(tDrone, tEnvironment)

    % first get measurements of walls
    afSensorReading = tDrone.GetWallMeasurements(tEnvironment);
    
    % check if any drone is close. Maybe easier to discretize the
    % environment and iterate cells?
    for iOtherDrone = 1:tEnvironment.iNumberOfDrones
        afDistance = tEnvironment.aafDronePos(1:2,iOtherDrone)- tDrone.afState(1:2);
        if afDistance == zeros(2,1)
            continue;
        end
        fDistMagnitude = sqrt(afDistance'*afDistance);
        
        % if another drone is closer than the wall, update sensor reading
        if fDistMagnitude <= tDrone.tSensorParam.fRange
            fDistMagnitude = Round2NearestMultiple(fDistMagnitude,tDrone.tSensorParam.fDeltaRange);
            fAngle = Round2NearestMultiple(atan2(afDistance(2),afDistance(1)), tDrone.tSensorParam.fDeltaAngle);
            if fAngle < 0
                fAngle = fAngle + 2*pi;
            end
            iAngleIndex = fAngle/tDrone.tSensorParam.fDeltaAngle + 1;
            if afSensorReading(iAngleIndex) >= fDistMagnitude
                afSensorReading(iAngleIndex) = fDistMagnitude;
            end
               
        end 
    
    end % end for iOtherDrone = 1:tEnvironment.iNumberOfDrones
    tDrone.afSensorReading = afSensorReading;
    
end


function fRounded = Round2NearestMultiple(fNumber, fMultiple)
    fRounded = fMultiple*round(fNumber/fMultiple);
end

