

function [tMap, aaiWalls, fEnvironmentArea] = CreateRoom(sTypeOfRoom, iRoomsizeX, iRoomsizeY, iPixelPerMeter)

    switch(sTypeOfRoom)
        case 'SquareRoom'
            innerX = [0 0 iRoomsizeX iRoomsizeX];
            innerY = [0 iRoomsizeY iRoomsizeY 0];
        case 'LRoom'
            innerX = [0 0 0.25*iRoomsizeX 0.25*iRoomsizeX iRoomsizeX iRoomsizeX];
            innerY = [0 iRoomsizeY iRoomsizeY 0.25*iRoomsizeY 0.25*iRoomsizeY 0];
        case 'CRoom'
            innerX = [0 0 iRoomsizeX iRoomsizeX 0.25*iRoomsizeX 0.25*iRoomsizeX iRoomsizeX iRoomsizeX];
            innerY = [0 iRoomsizeY iRoomsizeY 0.75*iRoomsizeY 0.75*iRoomsizeY 0.25*iRoomsizeY 0.25*iRoomsizeY 0];
        case 'HollowSquareRoom'
            outerX = [0 0 iRoomsizeX iRoomsizeX];
            outerY = [0 iRoomsizeY iRoomsizeY 0];
            innerX = [0.25*iRoomsizeX 0.25*iRoomsizeX 0.75*iRoomsizeX 0.75*iRoomsizeX];
            innerY = [0.25*iRoomsizeY 0.75*iRoomsizeY 0.75*iRoomsizeY 0.25*iRoomsizeY];
    end
    
    for iWall = 1:numel(innerX)-1
        aaiWalls(iWall,:) = [innerX(iWall) innerY(iWall) innerX(iWall+1) innerY(iWall+1)];
    end
    aaiWalls(end+1,:) = [innerX(end) innerY(end) innerX(1) innerY(1)];
    
    if strcmp(sTypeOfRoom,'HollowSquareRoom')
        tInnerPoly = polyshape(innerX, innerY);
        tOuterPoly = polyshape(outerX, outerY);
        tPolyWalls = xor(tInnerPoly, tOuterPoly);
        
        for iWall = 1:numel(outerX)-1
            aaiWalls(end+1,:) = [outerX(iWall) outerY(iWall) outerX(iWall+1) outerY(iWall+1)];
        end
        aaiWalls(end+1,:) = [outerX(end) outerY(end) outerX(1) outerY(1)];
        
    else
        tPolyWalls = polyshape(innerX, innerY);
        outerX = [];
        outerY = [];
    end
    fEnvironmentArea = area(tPolyWalls);
    
    tMap = binaryOccupancyMap(iRoomsizeX, iRoomsizeY, iPixelPerMeter);
    for iWall = 1:size(aaiWalls,1)
        if aaiWalls(iWall,1) > aaiWalls(iWall,3)
            aiTemp = aaiWalls(iWall,1);
            aaiWalls(iWall,1) = aaiWalls(iWall,3);
            aaiWalls(iWall,3) = aiTemp;
        elseif aaiWalls(iWall,2) > aaiWalls(iWall,4)
            aiTemp = aaiWalls(iWall,2);
            aaiWalls(iWall,2) = aaiWalls(iWall,4);
            aaiWalls(iWall,4) = aiTemp;
        end
        addWall(tMap, aaiWalls(iWall,1:4));
    end
end



function addWall(tMap, aiWall)
    fDelta = 1/tMap.Resolution;
    if aiWall(1) == aiWall(3) % if wall is vertical
        yPos = [aiWall(2)+fDelta/2:fDelta:aiWall(4)]';
        xPos = aiWall(1)*ones(numel(yPos),1);
    else % wall is horizontal
        xPos = [aiWall(1)+fDelta/2:fDelta:aiWall(3)]';
        yPos = aiWall(2)*ones(numel(xPos),1);
    end
    
    setOccupancy(tMap, [xPos yPos], ones(numel(xPos),1));
end